#include "xsnmp.h"
#include "math.h"

#define Debug	(Flags & (PROG_DEBUG_NET | PROG_DEBUG))

NETENTRY		*Netlist=(NETENTRY *)NULL;
int			Lastnet;

SPEEDCOLOR Speedcolors[]={
    {100000000, "purple"},
    { 16000000, "deepskyblue"},
    { 10000000, "gold"},
    {  4000000, "lightblue"},
    {  1544000, "cyan"},
    {   512000, "green"},
    {   128000, "orange"},
    {    56000, "red"},
    {0}
};

LINETYPE Linetypes[]={
    {100000000, LineSolid, 8},
    { 16000000, LineSolid, 4},
    { 10000000, LineSolid, 2},
    {  4000000, LineSolid, 1},
    {  1544000, LineDoubleDash, 8},
    {   512000, LineDoubleDash, 4},
    {   128000, LineDoubleDash, 2},
    {    56000, LineDoubleDash, 1},
    {0}
};

alloccolors() {
    register SPEEDCOLOR	*scp;
    XColor		exact;

    for(scp = Speedcolors ; scp->sc_speed ; scp++) {
	if(Flags & PROG_COLOR) {
	    if(XAllocNamedColor(Mydisplay, Cmap, scp->sc_name
				, &exact, &scp->sc_color) == 0) {
		printf("couldn't allocate color '%s'\n", scp->sc_name);
		scp->sc_color.pixel = BlackPixelOfScreen(Myscrptr);
	    }
	} else
	    scp->sc_color.pixel = BlackPixelOfScreen(Myscrptr);
    }

    /* Force an update */
    XClearArea(Mydisplay, XtWindow(Form), 0, 0, Formwidth, Formheight, True);
}

significantfrac(frac) int frac; {
    char		buffer[40];
    register char	*cptr;

    sprintf(buffer, "%d", frac);
    for(cptr = &buffer[strlen(buffer)] /* search for first non-zero */
	; cptr > buffer && (*cptr == '0' || *cptr == '\0')
	; cptr--) ;
    *++cptr = '\0';		/* null terminate */

    return(atoi(buffer));
}

char *speedtoletters(speed, nspeed, fracspeed)
int	speed, *nspeed, *fracspeed; {
    
    if(speed > 1000000) {
	*nspeed = speed / 1000000;
	*fracspeed = significantfrac(speed % 1000000);
	return("Mb");
    } else if(speed > 1000) {
	*nspeed = speed / 1000;
	*fracspeed = significantfrac(speed % 1000);
	return("Kb");
    } else {
	*nspeed = speed;
	*fracspeed = 0;
	return("b");
    }
}

u_long speedtocolor(speed) int speed; {
    register SPEEDCOLOR	*scp;
    XColor		exact;
    
    for(scp = Speedcolors ; scp->sc_speed ; scp++)
	if(speed >= scp->sc_speed)
	    return(scp->sc_color.pixel);
    
    /* Couldn't find it */
    if(Flags & PROG_REVERSE)
	return(WhitePixelOfScreen(Myscrptr));
    else
	return(BlackPixelOfScreen(Myscrptr));
}

drawlegend() {
    static short	firsttime=TRUE;
    register SPEEDCOLOR	*scp;
    register LINETYPE	*ltp;
    register int	y;
    char		buffer[60];
    char		*letters;
    int			speed, nspeed, fracspeed;

    if(firsttime) {
	alloccolors();
	firsttime = FALSE;
    }

    if(Flags & PROG_COLOR) {
	for(scp = Speedcolors , y = LEGENDY; scp->sc_speed ; scp++) {
	    letters = speedtoletters(scp->sc_speed, &nspeed, &fracspeed);
	    if(fracspeed != 0)
		sprintf(buffer, "%d.%d%sps", nspeed, fracspeed, letters);
	    else
		sprintf(buffer, "%d%sps", nspeed, letters);
	    XSetForeground(Mydisplay, Gc, speedtocolor(scp->sc_speed));
	    XDrawLine(Mydisplay, Lines, Gc, LEGENDX, y, LEGENDX + 20, y);
	    XDrawString(Mydisplay, Lines, Gc, LEGENDX + 30, y
			, buffer, strlen(buffer));
	    y += LEGENDLINEHEIGHT;
	}
    } else {
	for(ltp = Linetypes , y = LEGENDY; ltp->lt_speed ; ltp++) {
	    letters = speedtoletters(ltp->lt_speed, &nspeed, &fracspeed);
	    if(fracspeed != 0)
		sprintf(buffer, "%d.%d%sps", nspeed, fracspeed, letters);
	    else
		sprintf(buffer, "%d%sps", nspeed, letters);
	    if(Flags & PROG_REVERSE)
		XSetForeground(Mydisplay, Gc, WhitePixelOfScreen(Myscrptr));
	    else
		XSetForeground(Mydisplay, Gc, BlackPixelOfScreen(Myscrptr));
	    XSetLineAttributes(Mydisplay, Gc, ltp->lt_width
			       , ltp->lt_style, CapButt, JoinRound);
	    XDrawLine(Mydisplay, Lines, Gc, LEGENDX, y, LEGENDX + 20, y);
	    XDrawString(Mydisplay, Lines, Gc, LEGENDX + 30, y
			, buffer, strlen(buffer));
	    y += LEGENDLINEHEIGHT;
	}
    }
}

/*
 * Set line attributes when drawing a network line
 */
setlineattrib(netptr) NETENTRY *netptr; {
    register LINETYPE	*ltp;

    if(Flags & PROG_COLOR) {
	XSetForeground(Mydisplay, Gc, speedtocolor(netptr->nl_speed));
	return;
    } else {
	/* Set line type and width */
	if(Flags & PROG_REVERSE)
	    XSetForeground(Mydisplay, Gc, WhitePixelOfScreen(Myscrptr));
	else
	    XSetForeground(Mydisplay, Gc, BlackPixelOfScreen(Myscrptr));
	for(ltp = Linetypes ; ltp->lt_speed ; ltp++)
	    if(netptr->nl_speed >= ltp->lt_speed) {
		XSetLineAttributes(Mydisplay, Gc, ltp->lt_width
				   , ltp->lt_style, CapRound, JoinRound);
		return;
	    }
	
	XSetLineAttributes(Mydisplay, Gc, 1, LineOnOffDash
			   , CapButt, JoinRound);
	return;
    }
}

/*
 * Reset the minx, miny, maxx, and maxy values unless they were
 * set by the user.
 */
resetnets(netptr) NETENTRY *netptr; {

    if(netptr == (NETENTRY *)NULL) return;

    if(! (netptr->nl_flags & NET_PINNED)) {
	netptr->nl_minx = SMALL_INFINITY;
	netptr->nl_miny = SMALL_INFINITY;
	netptr->nl_maxx = 0;
	netptr->nl_maxy = 0;
	netptr->nl_startx = 0;
	netptr->nl_starty = 0;
	netptr->nl_endx = 0;
	netptr->nl_endy = 0;
    }

    if(netptr->nl_next) resetnets(netptr->nl_next);
}

/*
 * Allocate a new netentry
 */
NETENTRY *newnetentry(netaddr, speed)
u_long		netaddr;
int		speed; {
    NETENTRY		*netptr;
    struct in_addr	net;

    if(Debug) {
	net.s_addr = netaddr;
	printf("Making new networkentry for %s\n", inet_ntoa(net));
    }

    if((netptr = (NETENTRY *)malloc(sizeof(NETENTRY))) == (NETENTRY *)NULL) {
	dispnotice("Error in newnetentry", "Can't malloc memory!");
    } else {
	netptr->nl_next = (NETENTRY *)NULL; /* used by resetnets */
	netptr->nl_name = (char *)NULL;
	netptr->nl_net.s_addr = netaddr;
	netptr->nl_speed = speed;
	netptr->nl_flags = 0;
	netptr->nl_radius = 0;

	/* reset all network values to good values */
	resetnets(netptr);

	/* insert into network list */
	netptr->nl_next = Netlist;
	Netlist = netptr;
    }

    return(netptr);
}

freenetentry(netptr) NETENTRY *netptr; {

    if(!netptr) return;

    if(netptr->nl_name) free(netptr->nl_name);
    free(netptr);
}

NETENTRY *removenetentry(netptr) NETENTRY *netptr; {
    register NETENTRY	*nptr;
    
    if(Debug)
	printf("removenetentry called for net %s\n"
	       , inet_ntoa(netptr->nl_net));

    if(!netptr) return;

    nptr = netptr->nl_next;

    freenetentry(netptr);

    return(nptr);
}

/*
 * Net routines
 */
void drawnets() {
    register int	i, j, tmp;
    int			x, y;
    double		theta;
    int			net;
    HOSTENTRY		*hptr;
    IFENTRY		*ifptr;
    NETENTRY		*netptr;

    Lastnet = 0;

    /* Reset all net's min and max values (if we calculated them) */
    resetnets(Netlist);

    /* What networks do we have? */
    for(ifptr = Ifentries ; ifptr ; ifptr = ifptr->if_nextif) {
 	if(ifptr->if_addr.s_addr == LOCALHOST) continue;
	if(ifptr->if_net.s_addr == 0) continue;
	if(ifptr->if_flags & FL_DOWN) continue;	/* interface is down */
	j = 0;
	hptr = ifptr->if_host;
	XtSetArg(Args[j], XtNx, &hptr->hl_x), j++;
	XtSetArg(Args[j], XtNy, &hptr->hl_y), j++;
	XtSetArg(Args[j], XtNheight, &hptr->hl_height), j++;
	XtSetArg(Args[j], XtNwidth, &hptr->hl_width), j++;
	XtGetValues(hptr->hl_widget, Args, j);
	/* Set the x and y values at the center of the rectangle */
	hptr->hl_x += hptr->hl_width / 2;
	hptr->hl_y += hptr->hl_height / 2;

	/* Save the min. and max. values for each net */
	for(netptr = Netlist ; netptr ; netptr = netptr->nl_next) {
	    if(netptr->nl_net.s_addr == ifptr->if_net.s_addr) {
		if(Debug) {
		    printf("looking at %s (%s)\n"
			   , inet_ntoa(ifptr->if_net)
			   , ifptr->if_host->hl_name);
		}
		if(Debug)
		    printf("saving min and max as (%d,%d) and (%d,%d) netptr = 0x%x\n"
			   , netptr->nl_minx, netptr->nl_miny
			   , netptr->nl_maxx, netptr->nl_maxy, netptr);
		if(hptr->hl_x < netptr->nl_minx)
		    netptr->nl_minx = hptr->hl_x;
		if(hptr->hl_x > netptr->nl_maxx)
		    netptr->nl_maxx = hptr->hl_x;
		if(hptr->hl_y < netptr->nl_miny)
		    netptr->nl_miny = hptr->hl_y;
		if(hptr->hl_y > netptr->nl_maxy)
		    netptr->nl_maxy = hptr->hl_y;
		if(! (netptr->nl_flags & NET_SPEEDSET)
		   && ifptr->if_speed > netptr->nl_speed) {
		    if(Debug)
			printf("saving speed\n");
		    netptr->nl_speed = ifptr->if_speed;
		}
		break;
	    }
	}

	if(netptr) continue;

	/* Have a new network */
	if((netptr = newnetentry(ifptr->if_net.s_addr, ifptr->if_speed))
	   != (NETENTRY *)NULL) {
	    netptr->nl_minx = hptr->hl_x;
	    netptr->nl_maxx = hptr->hl_x;
	    netptr->nl_miny = hptr->hl_y;
	    netptr->nl_maxy = hptr->hl_y;
	} else {		/* error */
	    dispnotice("Error in drawing network lines"
		       , "Can't malloc memory!");
	    return;
	}
    }

    /* Now set the startx,y and endx,y */
    for(netptr = Netlist ; netptr ; netptr = netptr->nl_next) {
	if(Debug) {
	    printf("net %s: min=(%d,%d), max=(%d,%d) netptr=0x%x\n"
		   , netptr->nl_name
		   , netptr->nl_minx, netptr->nl_miny, netptr->nl_maxx
		   , netptr->nl_maxy, netptr);
	}

	/* Circular network */
	if((netptr->nl_flags & NET_CIRCLE)
	   && !(netptr->nl_flags & NET_PINNED)) {
	    netptr->nl_startx = (netptr->nl_maxx + netptr->nl_minx) / 2;
	    netptr->nl_starty = (netptr->nl_maxy + netptr->nl_miny) / 2;
	    continue;
	}
	
	/* Line network */
	if(netptr->nl_maxx - netptr->nl_minx
	   > netptr->nl_maxy - netptr->nl_miny) {	/* horizontal net */
	    netptr->nl_startx = netptr->nl_minx;
	    netptr->nl_endx = netptr->nl_maxx;
	    if(! (netptr->nl_flags & NET_PINNED)) {
		netptr->nl_starty = (netptr->nl_maxy
				     + netptr->nl_miny) / 2;
		netptr->nl_endy = netptr->nl_starty;
	    }
	    /* Make sure this is above the line of all rectangles involved */
	    for(hptr = Hostlist ; hptr ; hptr = hptr->hl_next) {
		if(! onnet(hptr, netptr->nl_net.s_addr)) continue;
		/* this one is on our net! */
		if(hptr->hl_y - hptr->hl_height/2 - NETSPACING
		   < netptr->nl_starty
		   && hptr->hl_y + hptr->hl_height/2 + NETSPACING
		   > netptr->nl_starty) {
		    netptr->nl_endy = netptr->nl_starty
			= hptr->hl_y - hptr->hl_height/2 - NETSPACING;
		    j = -1;	/* start over again */
		}
	    }
	} else {		/* vertical net */
	    netptr->nl_starty = netptr->nl_miny;
	    netptr->nl_endy = netptr->nl_maxy;
	    if(! (netptr->nl_flags & NET_PINNED)) {
		netptr->nl_startx = (netptr->nl_maxx
				     + netptr->nl_minx) / 2;
		netptr->nl_endx = netptr->nl_startx;
	    }
	    /* Make sure this is left of the line of all rectangles involved */
	    for(hptr = Hostlist ; hptr ; hptr = hptr->hl_next) {
		if(! onnet(hptr, netptr->nl_net.s_addr)) continue;
		/* this one is on our net! */
		if(hptr->hl_x - hptr->hl_width/2 - NETSPACING
		   < netptr->nl_startx
		   && hptr->hl_x + hptr->hl_width/2 + NETSPACING
		   > netptr->nl_startx) {
		    netptr->nl_endx = netptr->nl_startx
			= hptr->hl_x - hptr->hl_width/2 - NETSPACING;
		    j = -1;	/* start over again */
		}
	    }
	}
    }

    /* Now draw the nets */
    if(Flags & PROG_REVERSE)
	XSetForeground(Mydisplay, Gc, BlackPixelOfScreen(Myscrptr));
    else
	XSetForeground(Mydisplay, Gc, WhitePixelOfScreen(Myscrptr));
    XFillRectangle(Mydisplay, Lines, Gc, 0, 0, Formwidth, Formheight);

    drawlegend();

    checknets();		/* get rid of unneeded nets */
    for(netptr = Netlist ; netptr ; netptr = netptr->nl_next) {
	setlineattrib(netptr);
	if(! (netptr->nl_flags & NET_CIRCLE)) {
	    if(Debug)
		printf("Drawing a net line from (%d,%d) to (%d,%d)\n"
		       , (int)netptr->nl_startx
		       , (int)netptr->nl_starty, (int)netptr->nl_endx
		       , (int)netptr->nl_endy);
	    XDrawLine(Mydisplay, Lines, Gc, (int)netptr->nl_startx
		      , (int)netptr->nl_starty, (int)netptr->nl_endx
		      , (int)netptr->nl_endy);
	} else {		/* Let's do a circle! */
	    if(netptr->nl_radius <= 0)
 		netptr->nl_radius = DEFAULTRADIUS;
	    if(Debug)
		printf("Drawing a net circle center (%d, %d) radius %d\n"
		       , netptr->nl_startx, netptr->nl_starty
		       , netptr->nl_radius);
	    XDrawArc(Mydisplay, Lines, Gc
		     , netptr->nl_startx - netptr->nl_radius
		     , netptr->nl_starty - netptr->nl_radius
		     , netptr->nl_radius * 2, netptr->nl_radius * 2
		     , 0, 360 * 64);
	}
    }

    /* And draw the lines from the net to each system */
    for(hptr = Hostlist ; hptr ; hptr = hptr->hl_next)
	for(ifptr = hptr->hl_if ; ifptr ; ifptr = ifptr->if_next) {
	    if(ifptr->if_addr.s_addr == LOCALHOST) continue;
	    if(ifptr->if_net.s_addr == 0) continue;
	    if(ifptr->if_flags & FL_DOWN) continue; /* interface is down */
	    if((netptr = iftonet(ifptr->if_net.s_addr)) != (NETENTRY *)NULL) {
		setlineattrib(netptr);
		if(! (netptr->nl_flags & NET_CIRCLE)) {
		    if(netptr->nl_startx == netptr->nl_endx) {
			/* vertical */
			if(Debug) {
			    printf("Drawing a host line from (%d,%d) to (%d,%d) "
				   , netptr->nl_startx, hptr->hl_y, hptr->hl_x
				   , hptr->hl_y);
			    printf("for host '%s'\n", hptr->hl_name);
			}
			XDrawLine(Mydisplay, Lines, Gc, netptr->nl_startx
				  , hptr->hl_y, hptr->hl_x
				  , hptr->hl_y);
		    } else {
			/* horizontal */
			XDrawLine(Mydisplay, Lines, Gc, hptr->hl_x
				  , hptr->hl_y
				  , hptr->hl_x, netptr->nl_starty);
		    }
		} else /* circular net */ {
		    /* check for only one system on net */
		    if(netptr->nl_startx == hptr->hl_x
		       && netptr->nl_starty == hptr->hl_y) continue;

		    /* calculate the angle between center and host */
		    theta = atan(((double)netptr->nl_starty - hptr->hl_y)
				 / (hptr->hl_x - netptr->nl_startx));
		    if(Debug)
			printf("theta = %f\n", theta);
		    if(hptr->hl_x < netptr->nl_startx) {
			y = (sin(theta) * netptr->nl_radius)
			    + netptr->nl_starty;
			x = netptr->nl_startx
			    - (cos(theta) * netptr->nl_radius);
		    } else {
			y = netptr->nl_starty
			    - (sin(theta) * netptr->nl_radius);
			x = (cos(theta) * netptr->nl_radius)
			    + netptr->nl_startx;
		    }
		    XDrawLine(Mydisplay, Lines, Gc, hptr->hl_x, hptr->hl_y
			      , x, y);
		}
	    }
	}
    
    /* Force an update */
    XClearArea(Mydisplay, XtWindow(Form), 0, 0, Formwidth, Formheight, True);
    Flags &= ~PROG_KICKNET;	/* finished redrawing network */
}

checknets() {
    register int	i;
    register NETENTRY	*netptr, *nptr;

    if(! Netlist) return;	/* done */
    
    /* Remove null length nets from the beginning of the chain */
    while(Netlist
	  && !Netlist->nl_flags
	  && (Netlist->nl_minx > Netlist->nl_maxx
	      || Netlist->nl_miny > Netlist->nl_maxy)) {
	netptr = Netlist;
	Netlist = Netlist->nl_next;
	removenetentry(netptr);
    }

    /* Remove all other null length nets */
    for(netptr = Netlist
	; netptr && netptr->nl_next
	; netptr = netptr->nl_next) {
	if(netptr->nl_next->nl_flags) continue;	/* don't delete info. */
	if(netptr->nl_next->nl_minx > netptr->nl_next->nl_maxx
	   || netptr->nl_next->nl_miny > netptr->nl_next->nl_maxy) {
	    nptr = netptr->nl_next;
	    netptr->nl_next = netptr->nl_next->nl_next;
	    removenetentry(nptr);
	    continue;
	}
    }
}

NETENTRY *iftonet(address) u_long address; {
    register int	i;
    register NETENTRY	*netptr;
    
    for(netptr = Netlist ; netptr ; netptr = netptr->nl_next)
	if(netptr->nl_net.s_addr == address)
	    return(netptr);

    return((NETENTRY *)NULL);
}

onnet(hptr, net) HOSTENTRY *hptr; u_long net; {
    register IFENTRY	*ifptr;

    for(ifptr = hptr->hl_if
	; ifptr
	; ifptr = ifptr->if_next)
	if(ifptr->if_net.s_addr == net) return TRUE;

    return FALSE;
}

/*
 * Find the last host on a network:
 * If the network is horizontal, it's the one on the right
 * else if the network is vertical, it's the one on bottom.
 */
#define NET_HORIZONTAL	1
#define NET_VERTICAL	2
HOSTENTRY *lastnethost(netptr) NETENTRY *netptr; {
    register HOSTENTRY	*hptr;
    register IFENTRY	*ifptr;
    int			netmode;
    int			position;

    if(netptr->nl_startx == netptr->nl_endx)
	netmode = NET_VERTICAL;
    else
	netmode = NET_HORIZONTAL;

    for(ifptr = Ifentries , position = 0 ; ifptr ; ifptr = ifptr->if_nextif) {
	if(ifptr->if_addr.s_addr == LOCALHOST) continue;
	if(ifptr->if_addr.s_addr == 0) continue;
	if(ifptr->if_net.s_addr == netptr->nl_net.s_addr) {
	    if(netmode == NET_HORIZONTAL && ifptr->if_host->hl_x > position) {
		hptr = ifptr->if_host;
		position = ifptr->if_host->hl_x;
	    }
	    if(netmode == NET_VERTICAL && ifptr->if_host->hl_y > position) {
		hptr = ifptr->if_host;
		position = ifptr->if_host->hl_y;
	    }
	}
    }

    return(hptr);
}

addhosttonet(hptr) HOSTENTRY *hptr; {
    register int	i;
    register IFENTRY	*ifptr;
    HOSTENTRY		*lasthost;
    Position		x, y;
    NETENTRY		*netptr;

    if(hptr == (HOSTENTRY *)NULL) return;

    /* Figure out what network this host should be on */
    for(ifptr = hptr->hl_if ; ifptr ; ifptr = ifptr->if_next)
	for(netptr = Netlist ; netptr ; netptr = netptr->nl_next)
	    if(netptr->nl_net.s_addr == ifptr->if_net.s_addr) {
		/* Found one! - Now see what type of net this is */
		if(netptr->nl_starty == netptr->nl_endy) {
		    /* horizontal net - place on right */
		    lasthost = lastnethost(netptr);
		    x = lasthost->hl_x + lasthost->hl_width / 2;
		    y = lasthost->hl_y - lasthost->hl_height / 2;
		} else {
		    /* vertical net - place on bottom */
		    lasthost = lastnethost(netptr);
		    x = lasthost->hl_x - lasthost->hl_width / 2;
		    y = lasthost->hl_y + lasthost->hl_height / 2;
		}
		XtMoveWidget(hptr->hl_widget, x, y);
		gridhost(hptr);	/* position it wrt the invisible grid */
		Flags |= PROG_KICKNET; /* update network lines */
		return;
	    }


    /* Can't find any network so just put it here */
    XtMoveWidget(hptr->hl_widget, INITIAL_X, INITIAL_Y);
}

/*
 * Find what network runs through a given X and Y coord.
 */
NETENTRY *findnet(x, y) int x, y; {
    register int	i;
    register NETENTRY	*foundnet, *netptr;
    double		distx, disty;

    for(netptr = Netlist , foundnet = (NETENTRY *)NULL
	; netptr
	; netptr = netptr->nl_next) {
	if(netptr->nl_flags & NET_CIRCLE) {
	    distx = x - netptr->nl_startx;
	    disty = y - netptr->nl_starty;
	    if(sqrt((distx * distx) + (disty * disty)) - netptr->nl_radius
	       <= NETSELECTDIST) {
		if(! foundnet)
		    foundnet = netptr;
		else
		    dispnotice("Select net", "Ambiguous network selection");
	    }
	} else if(netptr->nl_startx == netptr->nl_endx) {
	    /* vertical net */
	    
	    if(y > netptr->nl_starty && y < netptr->nl_endy
	       && abs(x - netptr->nl_startx) < NETSELECTDIST) {
		if(! foundnet)
		    foundnet = netptr;
		else
		    dispnotice("Select net", "Ambiguous network selection");
	    }
	} else {
	    /* horizontal net */
	    
	    if(x > netptr->nl_startx && x < netptr->nl_endx
	       && abs(y - netptr->nl_starty) < NETSELECTDIST) {
		if(! foundnet)
		    foundnet = netptr;
		else
		    dispnotice("Select net", "Ambiguous network selection");
	    }
	}
    }

    return(foundnet);
}

/*
 * Assign a name to a network
 */
setnetname(netptr, name) NETENTRY *netptr; char *name; {

    if(netptr->nl_name) free(netptr->nl_name);

    /* copy it and assign it */
    netptr->nl_name = cpystr(name);
}

PARSENET *newparsenet(addr, incl) u_long addr; int incl; {
    register PARSENET	*pptr;

    if((pptr = (PARSENET *)malloc(sizeof(PARSENET)))
       == (PARSENET *)NULL) {
	fprintf(stderr, "Can't alloc %d bytes of memory! (newparsenet)\n"
		, sizeof(PARSENET));
	return((PARSENET *)NULL);
    }

    pptr->pan_addr = addr;
    pptr->pan_name = (char *)NULL;
    pptr->pan_startx = 0;
    pptr->pan_starty = 0;
    pptr->pan_endx = 0;
    pptr->pan_endy = 0;
    pptr->pan_speed = 0;
    pptr->pan_radius = 0;
    pptr->pan_centerx = 0;
    pptr->pan_centery = 0;
    pptr->pan_flags = incl;

    return(pptr);
}

freeparsenet(pptr) PARSENET *pptr; {

    if(pptr->pan_name) free(pptr->pan_name);

    free(pptr);

    return;
}

setnetline(nptr, startx, starty, endx, endy)
NETENTRY	*nptr;
int		startx, starty, endx, endy; {

    nptr->nl_startx = startx;
    nptr->nl_starty = starty;
    nptr->nl_endx = endx;
    nptr->nl_endy = endy;

    nptr->nl_flags &= ~NET_CIRCLE;
}

setnetcircle(nptr, centerx, centery, radius)
NETENTRY	*nptr;
int		centerx, centery, radius; {

    nptr->nl_startx = centerx;
    nptr->nl_starty = centery;
    if(radius > 0)
	nptr->nl_radius = radius;
    else
	nptr->nl_radius = DEFAULTRADIUS;

    nptr->nl_flags |= NET_CIRCLE;
}


innetlist(net) NETENTRY *net; {
    register NETENTRY	*nptr;

    for(nptr = Netlist ; nptr && nptr != net ; nptr = nptr->nl_next) ;

    if(nptr)
	return(TRUE);
    else
	return(FALSE);
}

char *nettypestring(nptr) NETENTRY *nptr; {

    if(nptr->nl_flags & NET_CIRCLE)
	return("circle");
    else
	return("line");
}

setnettype(nptr, type) NETENTRY *nptr; char *type; {

    if(sstrcmp(type, "line") == 0)
	nptr->nl_flags &= ~NET_CIRCLE;
    else if(sstrcmp(type, "circle") == 0)
	nptr->nl_flags |= NET_CIRCLE;
}
