#include "xsnmp.h"

/*
 * Append text to the end of a buffer and return a pointer to the new end.
 */
char *appendstr(string, buffer, size, endptr)
char	*string, **buffer;
int	*size;
char	**endptr; {
    register int	strsize;
    char		*oldbuf;
    
    /* verify arguments */
    if(string == (char *)NULL) string = "\0";
    if(*size < 0) return((char *)NULL);
    if(*buffer == (char *)NULL)
	if((*buffer = (char *)malloc(TEXTALLOCINC)) == (char *)NULL) {
	    printf("Error allocating initial text buffer area\n");
	    return((char *)NULL);
	} else {
	    **buffer = '\0';
	    *size = TEXTALLOCINC;
	    *endptr = *buffer;
	}
    if(*endptr < *buffer) return((char *)NULL);
    
    strsize = strlen(string) + 1;

    /* If we need to reallocate the text area, do so */
    if(*size - (*endptr - *buffer) < strsize) {
	/* must allocate new storage */
	oldbuf = *buffer;
	if((*buffer = (char *)realloc(*buffer, *size + TEXTALLOCINC))
	   == (char *)NULL) {
	    printf("Error reallocating text buffer area!\n");
	    return((char *)NULL);
	}
	*size += TEXTALLOCINC;
	/* calculate new endptr */
	*endptr = *buffer + (*endptr - oldbuf);
    }

    /* copy the new text in */
    bcopy(string, *endptr, strsize);
    *endptr += strsize-1;
    return(*endptr + strsize - 1);
}

char *skipspaces(cptr) char *cptr; {

    /* skip over leading spaces */
    while(*cptr && isspace(*cptr)) cptr++;
    
    return cptr;
}

normalizetext(text) char *text; {
    register char	*cptr;

    for(cptr = text ; *cptr ; cptr++)
	switch( *cptr ) {
	case '\012':		/* returns -> newlines */
	    *cptr = '\n';
	    break;

	case '\\':		/* look for \12 */
	    if(*(cptr+1) && *(cptr+1) == '1' && *(cptr+2) && *(cptr+2) == '2')
		replacetext(cptr, 3, '\n');
	}
}

movetext(cptr, distance) char *cptr; int distance; {
    register char	*cptr2;

    cptr2 = cptr + distance;

    if(distance > 0) {
	do {
	    *cptr++ = *cptr2;
	} while(*cptr2++);
    } else {
	do {
	    *cptr2++ = *cptr;
	} while(*cptr++);
    }
}

replacetext(cptr, distance, replacechar)
char	*cptr;
int	distance;
int	replacechar; {
    register char	*cptr2;
    char		replchar;

    /* problems with signed and unsigned chars on the stack */
    replchar = replacechar & 0377;

    *cptr++ = replchar;

    movetext(cptr, distance-1);
}

/*
 * Copy a string to a new location
 */

char *cpystr(string) char *string; {
    register char	*cp;

    if(string == (char *)NULL || *string == '\0') {
	if((cp = (char *)malloc(1)) == (char *)NULL) {
	    dispnotice("Error in cpystr", "Can't malloc memory");
	    cp = (char *)NULL;
	} else
	    *cp ='\0';
    } else if((cp = (char *)malloc(strlen(string) + 1)) == (char *)NULL) {
	dispnotice("Error in cpystr", "Can't malloc memory");
	cp = (char *)NULL;
    } else
	strcpy(cp, string);

    return(cp);
}

/*
 * Make up a char string of the time in days, hours, ...
 * given 100th's of seconds.
 */
#define A_DAY		(AN_HOUR * 24)
#define AN_HOUR		(A_MINUTE * 60)
#define A_MINUTE	(A_SECOND * 60)
#define A_SECOND	(1)

char *time2str(seconds) u_long seconds; {
    register		i;
    static char		buffer[80];
    char		*bufptr, *endptr;
    int			size;

    bufptr = (char *)NULL;
    endptr = (char *)NULL;
    size = 0;

    if(seconds >= A_DAY) {
	i = seconds / A_DAY;
	sprintf(buffer, "%d day%s ", i, (i > 1) ? "s " : " ");
	appendstr(buffer, &bufptr, &size, &endptr);
	seconds -= i * A_DAY;
    }

    if(seconds >= AN_HOUR) {
	i = seconds / AN_HOUR;
	sprintf(buffer, "%d hour%s ", i, (i > 1) ? "s " : " ");
	appendstr(buffer, &bufptr, &size, &endptr);
	seconds -= i * AN_HOUR;
    }

    if(seconds >= A_MINUTE) {
	i = seconds / A_MINUTE;
	sprintf(buffer, "%d minute%s ", i, (i > 1) ? "s " : " ");
	appendstr(buffer, &bufptr, &size, &endptr);
	seconds -= i * A_MINUTE;
    }

    if(seconds >= A_SECOND) {
	i = seconds / A_SECOND;
	sprintf(buffer, "%d second%s", seconds
		, (i > 1) ? "s " : " ");
	appendstr(buffer, &bufptr, &size, &endptr);
    }

    if(bufptr == (char *)NULL || *bufptr == '\0')
	appendstr("0 seconds", &bufptr, &size, &endptr);

    return(bufptr);
}

lowerstring(cptr) char *cptr; {
    register char	*cp;

    for(cp = cptr ; *cp ; cp++)
	if(isupper(*cp)) *cp = tolower(*cp);
}

/*
 * Return a good value even if we're called with a null pointer as one of the
 * strings.
 * NOTE: if ONE string is a null pointer, it doesn't return the correct value.
 */
sstrcmp(str1, str2) char *str1, *str2; {

    if(str1 && *str1 && str2 && *str2) return(strcmp(str1, str2));
    else if(!str1 || !*str1) {
	if(!str2 || !*str2) return(0);
	else return(1);
    } else return(-1);
}
